//
// $Id: module.pmod,v 1.44 2004/06/04 17:13:03 vida Exp $
//

#pike __REAL_VERSION__
constant __version=0.1;
constant __author="Marc Dirix <marc@electronics-design.nl";
constant __components=({"Public.pmod/Protocols.pmod/POP.pmod"});

class Protocol
{

  inherit Stdio.FILE : sock;
  string message;

  int readreturncode()
  {
    string code;

    message=sock::gets();
    if(!message) return 1;
    sscanf(message,"%3s",code);
    return code=="+OK" ? 0 : 1;
  }
  
  int command(string cmd)
  {
    sock::write(cmd+"\r\n");
    return readreturncode();
  }

  array(string) read_body_lines()
  {
    array(string) ret=({});
    string s;
    while(s = sock::gets())
    {
      if(s=="." || s==".\r") return ret;
      ret+=({s+"\n"});
    }
    error("Connection closed by server.\n");
  }

  
}

//!
class Client
{
  inherit Protocol;

  static private int cmd(string c, string|void comment)
  {
    int r = command(c);
    switch(r) {
    case 0:
      break;
    default:
      error( "POP: "+c+"\n"+(comment?"POP: "+comment+"\n":"")+
	     "POP: "+message+"\n" );
    }
    return r;
  }

  void create(void|string|Stdio.File server, int|void port)
  {
    if(!server)
    {
      // Lookup MX record here (Using DNS.pmod)
      object dns=master()->resolv("Protocols")["DNS"]->client();
      server=dns->get_primary_mx(gethostname());
    }

    if (objectp(server))
       assign(server);
    else
    {
       if(!port)
	  port = 110;

       if(!server || !connect(server, port))
       {
	 error("Failed to connect to POP server.\n");
       }
    }

    if(readreturncode())
      error("Connection refused by POP server.\n");

  }

  int auth_plain(string username, string password)
  {
    if(cmd("user "+username))
      return 1;
    if(cmd("pass "+password)) 
         return 1;
    return 0;
  }

  mapping list()
  {
    mapping list = ([]);
    if(cmd("LIST"))
      return ([]);
    foreach(read_body_lines(), string s)
    {
      int id,size;
      sscanf(s,"%d %d",id,size);
      list += ([id:size]);
    }
    return list;
  }

  array retr(int messageid)
  {
    if(cmd(sprintf("RETR %d",messageid)))
      return ({});
    return read_body_lines();
  }
  
  array top(int messageid, int lines)
  {
    if(cmd(sprintf("TOP %d %d",messageid,lines)))
      return ({});
    return read_body_lines();
  }

  int dele(int messageid)
  {
    return cmd(sprintf("DELE %d",messageid));
  }
  int quit()
  {
    return cmd("QUIT");
  }
  int noop()
  {
    return cmd("NOOP");
  }
};
